package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.constant.BusinessConstant;
import pps.core.space.entity.SpaceOptimizeMidPeriodView;
import pps.core.space.mapper.SpaceOptimizeMidPeriodViewMapper;
import pps.core.space.service.data.space_optimize_mid_period.GetSpaceOptimizeMidPeriodViewInput;
import pps.core.space.service.data.space_optimize_mid_period.GetSpaceOptimizeMidPeriodViewOutput;
import pps.core.space.service.data.space_optimize_mid_period.QuerySpaceOptimizeMidPeriodViewInput;
import pps.core.space.service.data.space_optimize_mid_period.QuerySpaceOptimizeMidPeriodViewOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 中短期间开优化模块
 *
 * @author ZWT
 * @date 2023/09/07
 */
@XService
@XText("中短期间开优化模块")
public class SpaceOptimizeMidPeriodService {

    /**
     * 中短期间开优化--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeMidPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("中短期间开优化--分页列表")
    public XPageResult<QuerySpaceOptimizeMidPeriodViewOutput> querySpaceOptimizeMidPeriodView(XContext context, QuerySpaceOptimizeMidPeriodViewInput input) {
        SpaceOptimizeMidPeriodViewMapper mapper = context.getBean(SpaceOptimizeMidPeriodViewMapper.class);
        SpaceOptimizeMidPeriodView record = XCopyUtils.copyNewObject(input, SpaceOptimizeMidPeriodView.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeMidPeriodView> list = mapper.selectList(record);
        PageInfo<SpaceOptimizeMidPeriodView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceOptimizeMidPeriodViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeMidPeriodViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> ouMap = ServiceUtil.getOuMapByOuIds(context, outputs.stream()
                    .map(QuerySpaceOptimizeMidPeriodViewOutput::getOuId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String lineName = ServiceUtil.getPowerLineNameById(context, input.getLineId());
            outputs.forEach(output -> {
                output.setLineName(lineName);
                output.setOuName(ouMap.get(output.getOuId()));
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
                output.setOptimizeStateValue(output.getOptimizeState().equals(0) ? "已优化" : "未优化");
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 中短期间开优化--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceOptimizeMidPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("中短期间开优化--详情")
    public XSingleResult<GetSpaceOptimizeMidPeriodViewOutput> getSpaceOptimizeMidPeriodView(XContext context, GetSpaceOptimizeMidPeriodViewInput input) {
        SpaceOptimizeMidPeriodViewMapper mapper = context.getBean(SpaceOptimizeMidPeriodViewMapper.class);
        SpaceOptimizeMidPeriodView record = new SpaceOptimizeMidPeriodView();
        XCopyUtils.copyObject(input, record);
        SpaceOptimizeMidPeriodView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceOptimizeMidPeriodViewOutput output = new GetSpaceOptimizeMidPeriodViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }
}